from datetime import datetime
import re
import os
from .common import detect_file_encoding

def main(input_file, output_path):
	encoding = detect_file_encoding(input_file)
	current_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
	output_file = os.path.join(output_path, f'tosogou_output_{current_time}.txt')

	outfile=""
	with open(input_file, 'r', encoding=encoding) as infile, open(output_file, 'w+', encoding=encoding) as outfile:
		outfile.write(
			"# 生成工具 https://github.com/whitewatercn/rimetool\n" +
			"# 生成时间 " + current_time + "\n" 
			# 搜狗输入法导入时，识别"---"会报错
			# +
			# "---\n"
		)
		for line in infile:
			content = line.strip()
			if content and not content.startswith('#') and not content.startswith('---'):
				parts = content.split('\t')
				if len(parts) == 3:
					output, input, sort = parts
					new_input = "'" + input.replace(' ', "'")
					new_output = output
					new_line_output = new_output.strip().replace('-', "") + '\n'
					# 没法在这里转,有可能报bug
					outfile.write(new_line_output)

	
	print(f"已生成文件 {os.path.abspath(outfile.name)}")

	# 重新以二进制模式读取文件内容
	with open(output_file, 'rb') as f:
		content = f.read()

		# 以 GBK 编码重新写入到一个新文件中
		gbk_output_file = os.path.join(output_path, f'tosogou_output_{current_time}_gbk.txt')
		with open(gbk_output_file, 'wb') as f:
			try:
				decoded_content = content.decode(encoding)
				encoded_content = decoded_content.encode('gbk', errors='ignore')
				f.write(encoded_content)
				print(f"已将文件内容转换为 GBK 编码并保存到 {os.path.abspath(gbk_output_file)}")
			except UnicodeDecodeError:
				print("解码原文件内容时出错，请检查文件编码。")



if __name__ == "__main__":
	main()


class GUIConfig:
    # Text for the Beian link
    ICP_BEIAN_TEXT = "晋ICP备2025058330号"
    
    # Website name and title
    WEBSITE_NAME = "你好👋"
    WEBSITE_TITLE = "医键通词库转换工具"
    BACKEND_VERSION = "1.8.2"



from datetime import datetime
import os
import sys
from .rimetool_core.utils import Epub_Processor, vcf
from .rimetool_core.utils import simple_english
from .rimetool_core.utils import simple_chinese
from .rimetool_core.utils import tosougou
import argparse

help_text = """

参数说明:

| 参数            | 说明      | 简化形式 |
| ------------- | ------- | ---- |
| --input-path  | 输入文件路径  | -i   |
| --output-path | 输出路径    | -o   |
| --tool        | 启用工具    | -t   |
| --mode        | 工具的详细功能 | -m   |
|               |         |      |

工具说明:

| 参数                    | 说明                                               | 备注                   |
| --------------------- | ------------------------------------------------ | -------------------- |
| --tool vcf            | 用于将联系人文件（.vcf）导出为rime词库                          |                      |
| --tool simple-english | 将单个词（如hello）或单个词组（如hello world）文件（.txt）导出为rime词库 | simple-english可简化为se |
| --tool simple-chinese | 将单个中文词组（如你好）文件（.txt）导出为rime词库                    | simple-chinese可简化为sc |
| --tool tosougou       | 将rime词库导出为搜狗txt词库                                |                      |
| --tool epub           | epub相关功能，需指定--mode参数                             |                      |

epub模式说明:

| 参数       | 模式                | 说明               |
| -------- | ----------------- | ---------------- |
| --mode 1 | epub_to_txt       | 将EPUB转换为纯文本      |
| --mode 2 | txt_to_short_long | 将文本转换为短句和长句      |
| --mode 3 | txt_to_rime       | 将文本转换为rime格式     |
| --mode 4 | epub_to_rime      | 完整的EPUB到rime转换流程 |

"""

# 定义模式映射
mode_choices = {
    '1': 'epub_to_txt',
    '2': 'txt_to_short_long',
    '3': 'txt_to_rime',
    '4': 'epub_to_rime',
    'epub_to_txt': 'epub_to_txt',
    'txt_to_short_long': 'txt_to_short_long',
    'txt_to_rime': 'txt_to_rime',
    'epub_to_rime': 'epub_to_rime'
}

def get_args_parser(add_help=True):
    # 检查是否有 'web' 子命令
    if len(sys.argv) > 1 and sys.argv[1] == 'web':
        # 通过 web 命令启动网页端
        parser = argparse.ArgumentParser(description="启动rimetool网页界面", add_help=add_help)
        parser.add_argument('command', choices=['web'], help='启动网页界面')
        parser.add_argument('--host', default='0.0.0.0', help='服务器主机地址 (默认: 0.0.0.0)')
        parser.add_argument('--port', default=5023, type=int, help='服务器端口 (默认: 5023)')
        parser.add_argument('--debug', action='store_true', help='启用调试模式')
        parser.add_argument('--log-dir', help='日志文件目录 (默认: 当前目录/rimetool/logs)')
        return parser
    else:
        # 具体功能的实现
        parser = argparse.ArgumentParser(description=help_text, add_help=add_help, formatter_class=argparse.RawTextHelpFormatter)
        parser.add_argument('--input-path', '-i', required=True, type=str)
        parser.add_argument('--output-path', '-o', default='./rimetool_output', type=str)
        parser.add_argument('--tool', '-t', required=True, choices=['vcf','simple-english','se','simple-chinese','sc','tosougou','epub','hello'], type=str)
        parser.add_argument('--mode', '-m', required=False, choices=list(mode_choices.keys()))
        return parser

def main(output_files=None, is_web=False):
    # 检查是否是 web 命令
    if len(sys.argv) > 1 and sys.argv[1] == 'web':
        # 启动网页界面
        parser = get_args_parser()
        args = parser.parse_args()
        
        try:
            # 直接运行 new_app.py
            import subprocess
            import os
            
            # 获取 new_app.py 的路径
            current_dir = os.path.dirname(os.path.abspath(__file__))
            new_app_path = os.path.join(current_dir, 'rimetool_gui', 'new_app.py')
            
            print(f"启动rimetool网页界面...")
            print(f"访问地址: http://{args.host}:{args.port}")
            print("按 Ctrl+C 停止服务器")
            
            # 设置环境变量
            env = os.environ.copy()
            env['FLASK_HOST'] = args.host
            env['FLASK_PORT'] = str(args.port)
            if args.debug:
                env['FLASK_DEBUG'] = '1'
            
            # 设置日志目录
            if args.log_dir:
                env['RIMETOOL_LOG_DIR'] = args.log_dir
            else:
                # 默认在当前工作目录创建 rimetool/logs
                default_log_dir = os.path.join(os.getcwd(), 'rimetool', 'logs')
                env['RIMETOOL_LOG_DIR'] = default_log_dir
            
            # 运行 new_app.py
            subprocess.run([sys.executable, new_app_path], env=env, cwd=current_dir)
            
        except KeyboardInterrupt:
            print("\n网页服务器已停止")
        except Exception as e:
            print(f"启动网页服务器失败: {e}")
            sys.exit(1)
        return
    
    # 原有的处理逻辑
    current_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
    parser = get_args_parser()
    args = parser.parse_args()
    name = ""
    if not os.path.exists(args.output_path):
        os.makedirs(args.output_path)
    os.makedirs(args.output_path, exist_ok=True)
    if args.tool == 'vcf':
        name = vcf.main(args.input_path, args.output_path, is_web)
    elif args.tool in ['simple-english', 'se']:
        name = simple_english.main(args.input_path, args.output_path, is_web)
    elif args.tool in ['simple-chinese', 'sc']:
        name = simple_chinese.main(args.input_path, args.output_path, is_web)
    elif args.tool == 'tosougou':
        name = tosougou.main(args.input_path, args.output_path, is_web)
    elif args.tool == 'epub':
        output_dir = args.output_path
        output_files = {
            'clean': os.path.join(output_dir, "epub转txt.txt"),
            'short': os.path.join(output_dir, "短句拆分.txt"),
            'long': os.path.join(output_dir, "长句拆分.txt")
        }
        print(output_files)
        processor = Epub_Processor.EpubProcessor(args.input_path, output_dir, output_files,current_time, is_web)
        
        mode = mode_choices[args.mode]
        if mode == 'epub_to_txt':
            output_file = processor.epub_to_txt()
            if is_web:
                return output_file
        elif mode == 'txt_to_short_long':
            output_files = processor.txt_to_short_long(args.input_path, output_files)
            if is_web:
                return output_files
        elif mode == 'txt_to_rime':
            output_files = processor.txt_to_rime_all(args.input_path,output_files)
            if is_web:
                return output_files
        elif mode == 'epub_to_rime':
            output_files = processor.epub_to_rime(output_files)
            if is_web:
                return output_files
        else:
            raise ValueError('请选择正确的EPUB处理模式')
    else:
        raise ValueError('请选择正确的工具。')
    return name

def main_with_args(args_list):
    """
    用于在GUI中调用
    """
    original_argv = sys.argv
    sys.argv = [''] + args_list  # 设置命令行参数
    try:
        parser = get_args_parser()
        args = parser.parse_args(args_list)
        output_files = None
        print("args.tool:"+args.tool)
        if args.tool == 'epub':
            output_dir = args.output_path
            output_files = {
                'clean': os.path.join(output_dir, "epub转txt.txt"),
                'short': os.path.join(output_dir, "短句拆分.txt"),
                'long': os.path.join(output_dir, "长句拆分.txt")
            }
            sys.argv = [''] + args_list 
        
        # 标记这是从web界面调用的
        result = main(output_files, is_web=True)
        return result
    finally:
        sys.argv = original_argv  # 恢复原始的命令行参数

if __name__ == "__main__":
    main()


import ebooklib
from ebooklib import epub
from bs4 import BeautifulSoup
import re
import os
from pypinyin import lazy_pinyin
from datetime import datetime
from .common import detect_file_encoding, replace_roman_with_chinese

from .Pinyin_Processor import pinyin_process
import warnings 
# import subprocess

warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.simplefilter(action='ignore', category=UserWarning)


# current_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')

class EpubProcessor:
    """
    处理EPUB文件的类，包含多种方法，如epub->txt，txt->长短句，txt和长短句->rime
    """
    def __init__(self, input_path, output_path, output_files, current_time, is_web=False):
        """初始化处理器
        Args:
            input_path: EPUB文件本身的路径
            output_path: 输出文件路径，这是路径也就是文件夹，不是文件
            output_files: 输出文件路径，这是txt文件们：原版、短句、长句
            is_web: 是否是通过Web界面调用
        """
        self.input_path = input_path # 这是EPUB文件本身的路径 .replace('\\', '/') 
        self.output_path = output_path # 这是路径也就是文件夹，不是文件  .replace('\\', '/')
        self.output_files = output_files # 这是txt文件们：原版、短句、长句
        self.content = ""
        self.processed_content = []
        self.current_time = current_time
        self.is_web = is_web
        self.web_output_files = []  # 用于收集需要在Web界面下载的文件
    def read_epub(self):
        """从EPUB文件中读取内容"""
        tmp=''
        book = epub.read_epub(self.input_path)
        for item in book.get_items():
            if item.get_type() == ebooklib.ITEM_DOCUMENT:
                # self.content += item.get_body_content().decode('utf-8')
                content = item.get_body_content().decode('utf-8')
    
    # def extract_sections(self):
        # """提取所有章节内容"""
                sections = re.findall(r'<.*id=\"CHP.*?>.*</.*?>',content, re.DOTALL)
                
                for section in sections:
                    tmp+=section
                    tmp+='\n'
        return tmp
    
    def process_html(self, html_content):
        """处理HTML内容，提取特定标签"""
        soup = BeautifulSoup(html_content, 'html.parser')
        
        # 提取h3标签,标题
        h3_tags = soup.find_all('div', class_='h3', id=re.compile(r'CHP\d+'))
        for tag in h3_tags:
            self.processed_content.append(str(tag))
        
        
        # 提取p标签
        p_tags = soup.find_all('div', class_='pCls')
        for tag in p_tags:
            self.processed_content.append(str(tag))
    
    def clean_html(self):
        """清理HTML标签，只保留文本内容"""
        final_soup = BeautifulSoup('\n'.join(self.processed_content), 'html.parser')
        for div_tag in final_soup.find_all('div'):
            div_tag.unwrap()
        for a_tag in final_soup.find_all('a'):
            a_tag.unwrap()
        for b_tag in final_soup.find_all('b'):
            b_tag.unwrap()
        for br_tag in final_soup.find_all('br'):
            br_tag.unwrap()
        soup = str(final_soup)
        soup=soup.replace('　','')
        soup=soup.replace('1\n2','')
        soup=soup.replace('\n\n\n','\n')
        soup=soup.replace('\n\n ','\n')
        soup=soup.replace('        ','')

        return soup
    
    def save_output(self, content):
        """保存处理结果到文件"""
        txt_file = os.path.join(self.output_path, f"epub转txt_{self.current_time}.txt")
        with open(txt_file, 'w', encoding='utf-8') as f:
            f.write(content)
        if self.is_web:
            self.web_output_files.append(txt_file)
        return txt_file

    def split_into_short_sentences(self, input_path, output_dir):
        # 添加时间戳到输出文件名
        output_path = f"{os.path.splitext(output_dir)[0]}_{self.current_time}{os.path.splitext(output_dir)[1]}"
        
        # 读取清理后的内容
        epub_txt_file = input_path
        with open(epub_txt_file, 'r', encoding='utf-8') as f:
            file_content = f.read()
        
        # 拆分短句
        split_content = re.split(r'[，。！？；：、\s\n]+', file_content)
        
        # 保存结果
        with open(output_path, 'w', encoding='utf-8') as output_file:
            for item in split_content:
                if item:  # 避免写入空字符串
                    output_file.write(item + '\n')
        
        if self.is_web:
            self.web_output_files.append(output_path)
        
        return output_path
    
    # 长句不需要删除标点，输出时依然需要输出标点，如
    # ✅上四味，以水八升，先煮蜀漆、麻黄，
    # ❌上四味以水八升先煮蜀漆麻黄

    def split_into_long_sentences(self, input_path, output_dir):
        """将内容拆分成无标点的长句
        Args:
            output_path: 输出文件路径
        """
        # 添加时间戳到输出文件名
        output_path = f"{os.path.splitext(output_dir)[0]}_{self.current_time}{os.path.splitext(output_dir)[1]}"
        
        # 读取清理后的内容
        epub_txt_file = input_path
        with open(epub_txt_file, 'r', encoding='utf-8') as f:
            file_content = f.read()
        
        # 保存结果
        with open(output_path, 'w', encoding='utf-8') as output_file:
            
            # output_file.write(
            #     "# 生成工具 https://github.com/whitewatercn/rimetool\n" +
            #     "# 生成时间 " + current_time + "\n" +
            #     "---\n"
            # )
            for item in file_content.split('\n'):
                if item:  # 避免写入空字符串
                    cleaned_item = item
                    output_file.write(cleaned_item + '\n')
        
        if self.is_web:
            self.web_output_files.append(output_path)
        
        return output_path

    def to_rime(self, input_file, output_name):
        """txt->rime
        Args:
            input_file: 输入文件路径
            output_name: 输出文件名（不含扩展名）
        """
        output_file = os.path.join(self.output_path, f'{output_name}.dict.yaml')
        encoding = detect_file_encoding(input_file)

        with open(input_file, 'r', encoding=encoding) as infile, \
             open(output_file, 'w', encoding='utf-8') as outfile:
            outfile.write(
                "# 生成工具 https://github.com/whitewatercn/rimetool\n" +
                "# 生成时间 " + self.current_time + "\n" +
                "---\n"
            )
            # 处理每一行
            for line in infile:
                content = line.strip()
                if not content:  # 跳过空行
                    continue
                if content.startswith('#') or content == '---':
                    outfile.write(f"{content}\n")
                else:
                    # 将罗马数字转换为中文数字
                    content = replace_roman_with_chinese(content)
                    # 删除所有的 '-'
                    content = content.replace('-', '')
                    pinyin = ''
                    # 生成拼音
                    pinyin = pinyin.join(pinyin_process(content))
                    # 写入rime格式
                    outfile.write(f"{content}\t{pinyin}\t1\n")
        
        print(f"已生成rime格式文件: {os.path.abspath(output_file)}")
        
        if self.is_web:
            self.web_output_files.append(output_file)
        
        return output_file
    
    
    def epub_to_rime(self, output_files):
        """
        一键直达 epub->rime
        """
        input_path = self.epub_to_txt()
        result_files = self.txt_to_rime_all(input_path, output_files)
        
        if self.is_web:
            return self.web_output_files
        return result_files

    def txt_to_rime_all(self, input_path, output_files):
        """
        一键直达 txt->长短句的rime
        """
        short_path = self.txt_to_short_long(input_path, output_files)
        rime_files = self.txt_short_long_to_rime(output_files)
        
        if self.is_web:
            return self.web_output_files
        return rime_files


    def epub_to_txt(self):
        """
        第一个部分: epub->txt，简单地读取epub文件，转成一个没有各种标签（img，h1等）的txt文件
        """
        print("\n*** 第一部分: epub->txt ***\n")
        # 第一步：读取EPUB文件并处理格式
        print("读取EPUB文件...")
        try:
            sections = self.read_epub()
            print(f"EPUB文件读取成功，内容长度: {len(sections)}")
        except Exception as e:
            print(f"读取EPUB文件失败: {e}")
            return None
        # print("步骤2: 提取章节内容...")
        # sections = self.extract_sections()
        print("处理HTML内容...")
        self.process_html(sections)
        # print("步骤4: 清理HTML标签...")
        final_content = self.clean_html()
        # print("步骤5: 保存结果...")
        self.content = final_content
        output_file = self.save_output(final_content)
        
        print(f"epub->txt 处理完成！结果已保存到: {output_file}")
        return output_file

    def txt_to_short_long(self, input_path, output_files):
        """
        第二个部分: 将txt文件转成短词组和长词组词库
        """
        print("\n*** 第二部分: 将txt文件转成短词组和长词组词库 ***\n")
        # 第二步：执行短句拆分
        print("开始短句拆分...")
        short_path = self.split_into_short_sentences(input_path, output_files['short'])
        print(f"短句拆分完成，结果保存在: {output_files['short']}")

        # 第三步：执行长句拆分
        print("开始长句拆分...")
        long_path = self.split_into_long_sentences(input_path, output_files['long'])
        print(f"长句拆分完成，结果保存在: {output_files['long']}")
        
        result = [short_path, long_path]
        if self.is_web:
            return result
        return result

    def txt_short_long_to_rime(self, output_files):
        """
        第三个部分: 将txt原版、短词组、长词组词库转成rime格式
        """
        print("\n*** 第三部分: 将txt原版、短词组、长词组词库转成rime格式***\n")
        print("开始生成rime格式文件...")
        
        output_short_time = f"{os.path.splitext(output_files['short'])[0]}_{self.current_time}{os.path.splitext(output_files['short'])[1]}"
        output_long_time = f"{os.path.splitext(output_files['long'])[0]}_{self.current_time}{os.path.splitext(output_files['long'])[1]}"
        
        short_rime = self.to_rime(output_short_time, "short_rime"+self.current_time)
        long_rime = self.to_rime(output_long_time, "long_rime"+self.current_time)
        
        result = [short_rime, long_rime]
        if self.is_web:
            return result
        return result

    def replace_pinyin(self, input_path, output_path):
        return
        """替换拼音，只将"俞"字对应的"yu"替换为"shu"，其他位置的"yu"保持不变"""
        with open(input_path, 'r', encoding='utf-8') as infile, \
             open(output_path, 'w', encoding='utf-8') as outfile:
            # 替换俞的拼音
            for line in infile:
                parts = line.strip().split('\t')
                if len(parts) == 3:
                    sentence, pinyin_str, label = parts
                    modified_pinyin = self.replace_pinyin2(sentence, pinyin_str)
                    outfile.write(f"{sentence}\t{modified_pinyin}\t{label}\n")

    def replace_pinyin2(sentence: str, pinyin_str: str) -> str:
        return
        hanzi_list = list(sentence)
        pinyin_list = pinyin_str.split()
        
        # 生成汉字的拼音，保留格式（不带声调）
        generated_pinyin = lazy_pinyin(sentence)
        generated_pinyin = [item[0] for item in generated_pinyin]
        
        # 替换俞的拼音
        for i, char in enumerate(hanzi_list):
            if char == '俞' and pinyin_list[i] == 'yu':
                pinyin_list[i] = 'shu'
        
        return ' '.join(pinyin_list)



import os
from pypinyin import lazy_pinyin
from datetime import datetime
from .encoding_test import detect_file_encoding
from .roman_to_chinese import roman_to_chinese



def main(input_file, output_path, is_web=False):
	# 确保文件编码正确读入，并在输出时转为gbk
	encoding = detect_file_encoding(input_file)
	
	current_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
	output_file = os.path.join(output_path, f'simple_chinese_output_{current_time}.dict.yaml')
	with open(input_file, 'r', encoding=encoding) as infile, open(output_file, 'w', encoding='utf-8') as outfile:
		outfile.write(
			"# 生成工具 https://github.com/whitewatercn/rimetool\n" +
			"# 生成时间 " + current_time + "\n" +
			"---\n"
		)
		# 遍历输入文件的每一行

		for line in infile:
			content = line
			words = content.rstrip('\n').split('\t')
			new_line = words[0]
			pinyin_line = roman_to_chinese(new_line) # 将罗马转换为中文
			pinyin_line = pinyin_line.replace('-', '')  # 删除所有的 '-'
			pinyin = ' '.join(lazy_pinyin(pinyin_line))
			# 在行的内容后面加一个tab，然后加上它的拼音，再加一个tab，然后加上数字1
			new_line_with_pinyin = new_line + '\t' + pinyin + '\t1\n'
			outfile.write(new_line_with_pinyin)
		print(f"已生成文件 {os.path.abspath(outfile.name)}")
		
    # 返回文件名，用于web下载
	filename = os.path.basename(output_file)
	if is_web:
		return output_file
	return filename

if __name__ == "__main__":
    main()